/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */

package org.apache.tuscany.sca.itest.sdodatabinding;

import org.osoa.sca.annotations.Reference;
import org.osoa.sca.annotations.Service;

import org.apache.tuscany.sca.itest.databinding.types.PersonType;
import org.apache.tuscany.interop.AttributeComplexType;
import org.apache.tuscany.interop.AttributeReferenceComplexType;
import org.apache.tuscany.interop.SimpleTypeWithAbstractComplexType;
import org.apache.tuscany.interop.SimpleTypeWithNameComplexType;
import org.apache.tuscany.interop.ComplexTypeWithContentType;

/**
 * The automatically generated service that simply forwards data objects on to 
 * the GreeterService and passes back the responses
 *
 * @version $Rev: 508831 $ $Date: 2007-02-18 00:12:31 +0000 (Sun, 18 Feb 2007) $
 */
@Service(GreeterService.class)
public class GreeterServiceClientImpl implements GreeterService {

    /**
     * the reference the the GreeterService that will
     * change each data object slightly and return it
     */
    private GreeterService service;

    @Reference
    public void setGreeterService(GreeterService service) {
        this.service = service;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.tuscany.sca.itest.sdodatabinding.GreeterService#greetPersonType(PersonType param)
     */
    public PersonType greetPersonType(PersonType param) {
        return service.greetPersonType(param);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.tuscany.sca.itest.sdodatabinding.GreeterService#greetAttributeComplexType(AttributeComplexType param)
     */
    public AttributeComplexType greetAttributeComplexType(AttributeComplexType param) {
        return service.greetAttributeComplexType(param);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.tuscany.sca.itest.sdodatabinding.GreeterService#greetAttributeReferenceComplexType(AttributeReferenceComplexType param)
     */
    public AttributeReferenceComplexType greetAttributeReferenceComplexType(AttributeReferenceComplexType param) {
        return service.greetAttributeReferenceComplexType(param);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.tuscany.sca.itest.sdodatabinding.GreeterService#greetSimpleTypeWithAbstractComplexType(SimpleTypeWithAbstractComplexType param)
     */
    public SimpleTypeWithAbstractComplexType greetSimpleTypeWithAbstractComplexType(SimpleTypeWithAbstractComplexType param) {
        return service.greetSimpleTypeWithAbstractComplexType(param);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.tuscany.sca.itest.sdodatabinding.GreeterService#greetSimpleTypeWithNameComplexType(SimpleTypeWithNameComplexType param)
     */
    public SimpleTypeWithNameComplexType greetSimpleTypeWithNameComplexType(SimpleTypeWithNameComplexType param) {
        return service.greetSimpleTypeWithNameComplexType(param);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.tuscany.sca.itest.sdodatabinding.GreeterService#greetComplexTypeWithContentType(ComplexTypeWithContentType param)
     */
    public ComplexTypeWithContentType greetComplexTypeWithContentType(ComplexTypeWithContentType param) {
        return service.greetComplexTypeWithContentType(param);
    }
}
